#!/bin/bash

set -e

: ${CLICK_SDK_ARCH:?"Need to set CLICK_SDK_ARCH non-empty"}
: ${CLICK_SDK_FRAMEWORK:?"Need to set CLICK_SDK_FRAMEWORK non-empty"}
SERIES=utopic
CLICK_CHROOT_NAME=click

echo -e "\033[1;32m [ ${CLICK_SDK_ARCH} ] Bootstrapping development environment for Telegram\033[0m"

# Point this to a feature branch, if you need to test against non-trunk.
LIBQTELEGRAM_BRANCH=lp:libqtelegram
echo -e "           \033[1;32mBranching\033[1;33m ${LIBQTELEGRAM_BRANCH}\033[1;32m and updating library files in the source tree\033[0m"
rm -rf library
bzr branch $LIBQTELEGRAM_BRANCH library

update() {
    click chroot -a ${CLICK_SDK_ARCH} -f ${CLICK_SDK_FRAMEWORK} -n ${CLICK_CHROOT_NAME} upgrade
    click chroot -a ${CLICK_SDK_ARCH} -f ${CLICK_SDK_FRAMEWORK} -n ${CLICK_CHROOT_NAME} maint apt-get install -y \
    libssl-dev:${CLICK_SDK_ARCH} \
    libsqlite3-dev:${CLICK_SDK_ARCH} \
    libmediainfo-dev:${CLICK_SDK_ARCH} \
    libthumbnailer-dev:${CLICK_SDK_ARCH} \
    pkg-config-arm-linux-gnueabihf\
    qtdeclarative5-ubuntu-telephony-phonenumber0.1 \
    qml-module-qt-labs-settings \
    intltool \
    bzr \
    --no-install-recommends
}

if ! click chroot -a ${CLICK_SDK_ARCH} -f ${CLICK_SDK_FRAMEWORK} -n ${CLICK_CHROOT_NAME} exists
then
    echo -e "\033[1;31m [ ${CLICK_SDK_ARCH} ] Creating click chroot " \
        "${CLICK_SDK_FRAMEWORK}-${CLICK_SDK_ARCH} and installing dependencies\033[0m"
    sudo /usr/share/qtcreator/ubuntu/scripts/click_create_target \
            ${CLICK_SDK_ARCH} ${CLICK_SDK_FRAMEWORK} $SERIES $CLICK_CHROOT_NAME
    update
fi

if [ "$1" == "update" ]; then
    echo -e "\033[0;31m [ ${CLICK_SDK_ARCH} ] Updating click chroot\033[0m"
    update
fi

echo -e "\033[1;32m [ ${CLICK_SDK_ARCH} ] Bootstrapping complete.\033[0m"
