import QtQuick 2.0
import Telegram 0.1

Item {
    function getMediaTypeString(mediaType) {
        switch (mediaType) {
        case TLMessageMedia.TypeMessageMediaEmpty:
            return "Text";
        case TLMessageMedia.TypeMessageMediaPhoto:
            return "Photo";
        case TLMessageMedia.TypeMessageMediaVideo:
            return "Video";
        case TLMessageMedia.TypeMessageMediaGeo:
            return "Location";
        case TLMessageMedia.TypeMessageMediaContact:
            return "Contact";
        case TLMessageMedia.TypeMessageMediaAudio:
            return "Audio";
        case TLMessageMedia.TypeMessageMediaDocument:
            return "Document";
        case TLMessageMedia.TypeMessageMediaUnsupported:
            return "Unknown";
        default:
            return "Text";
        }
    }

    function getActionMessageText(outgoing, senderName, actionType,
                                  actionUser, actionTitle, actionThumbnail) {
        actionUser = actionUser.trim();
        var displayName = senderName;
        if (outgoing) displayName = i18n.tr("You");

        if (actionUser == "") {
            // TODO karni: We should be still able to display deleted user's name.
            // Why is a DELETED user's actionUser == " "?
            actionUser = i18n.tr("DELETED");
        }

        switch (true) {
        case actionType == TLMessageAction.TypeMessageActionEmpty:
            return "";
        case actionType == TLMessageAction.TypeMessageActionChatCreate: {
            return i18n.tr("%1 created %2").arg(displayName).arg(actionTitle);
        }
        case actionType == TLMessageAction.TypeMessageActionChatAddUser: {
            if (senderName == actionUser) {
                return i18n.tr("%1 rejoined the group").arg(displayName);
            } else {
                return i18n.tr("%1 added %2").arg(displayName).arg(actionUser);
            }
        }
        case actionType == TLMessageAction.TypeMessageActionChatDeleteUser: {
            if (senderName == actionUser) {
                return i18n.tr("%1 left the group").arg(displayName);
            } else {
                return i18n.tr("%1 removed %2").arg(displayName).arg(actionUser);
            }
        }
        case actionType == TLMessageAction.TypeMessageActionChatEditTitle:
            return i18n.tr("%1 changed group name to %2").arg(displayName).arg(actionTitle);
        case actionType == TLMessageAction.TypeMessageActionChatEditPhoto:
            return i18n.tr("%1 changed the group photo").arg(displayName); // TODO actionThumbnail
        case actionType == TLMessageAction.TypeMessageActionChatDeletePhoto:
            return i18n.tr("%1 removed the group photo").arg(displayName);
        default:
            console.log("Unhandled action: " + actionType);
            return "";
        }
    }
}
