import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import "TelegramColors.js" as TelegramColors

MouseArea {

    property alias image: image.source
    property alias title: title.text
    property string lastSeen
    property bool isOnline: false
    property bool selected: false

    id: listitem
    width: parent.width
    height: units.gu(8)

    Rectangle {
        id: background
        anchors.fill: parent
        color: listitem.selected ? TelegramColors.blue
                   : listitem.pressed ? TelegramColors.list_pressed : TelegramColors.white
    }

    UbuntuShape {
        id: imageShape
        anchors {
            top: parent.top
            topMargin: units.dp(4)
            left: parent.left
            leftMargin: units.gu(2)
            bottom: parent.bottom
            bottomMargin: units.dp(4)
            rightMargin: units.gu(1)
        }
        width: height

        image: Image {
            id: image
            anchors.fill: parent
            antialiasing: true
            asynchronous: true
            fillMode: Image.PreserveAspectFit
        }
    }

    Text {
        id: title
        anchors {
            top: parent.top
            topMargin: units.gu(1)
            left: imageShape.right
            leftMargin: units.gu(1.5)
            right: parent.right
            rightMargin: units.gu(1)
        }
        verticalAlignment: TextInput.AlignVCenter

        font.pixelSize: FontUtils.sizeToPixels("large")
        color: TelegramColors.black
        text: i18n.tr("Contact")
    }

    Text {
        id: subtitle
        anchors {
            top: title.bottom
            left: title.left
            bottom: parent.bottom
            bottomMargin: units.dp(4)
            right: parent.right
        }
        verticalAlignment: TextInput.AlignVCenter

        font.pixelSize: FontUtils.sizeToPixels("medium")
        color: isOnline ? TelegramColors.dark_blue : TelegramColors.grey
        text: i18n.tr("Last seen")
    }

    ListItem.ThinDivider {
        id: divider
        anchors {
            bottom: parent.bottom
            left: parent.left
            leftMargin: units.gu(2)
            right: parent.right
            rightMargin: units.gu(2)
        }
    }

    onLastSeenChanged: updateLastSeen(lastSeen)

    onIsOnlineChanged: updateLastSeen(lastSeen)

    function updateLastSeen(lastSeen) {
       if (isOnline) {
            subtitle.text = i18n.tr("online");
        } else {
            // TRANSLATORS: the argument refers to a time
            subtitle.text = i18n.tr("last seen %1").arg(lastSeen);
        }
    }
}
