import QtQuick 2.0
import QtGraphicalEffects 1.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "listitems"
import Telegram 0.1
import "TelegramColors.js" as Color
import "../js/ba-linkify.js" as BaLinkify
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time

ListItemWithActions {
    id: messageDelegate

    property bool outgoing: false
    property bool isSent: false
    property bool isRead: false

    property bool isNewDay: false
    property string newDayText: Time.formatSection(messageDate * 1000)

    property bool isAction: false
    property string actionType: TLMessageAction.TypeMessageActionEmpty
    property string actionTitle: ""
    property string actionUser: ""
    property string actionThumbnail: ""

    property int messageId: 0
    property string message: ""
    property int messageDate: 0
    property int mediaType: 0
    property string senderColor: ""
    // Used in PreviewPage title: From <firstName>
    property string senderDisplayName: ""
    property string senderName: ""
    property string senderImage: ""
    property string time: ""
    property string thumbnail: ""
    property bool hasThumbnail: thumbnail != ""
    property string photo: ""
    property string video: ""

    property bool isVideo: mediaType === TLMessageMedia.TypeMessageMediaVideo
    property bool isPhoto: mediaType === TLMessageMedia.TypeMessageMediaPhoto
    property bool isPhotoOrVideo: isVideo || isPhoto
    property bool needsDownload: isVideo && model.video === "" || isPhoto && model.photo === ""
    property bool isDownloading: false
    property variant progress: undefined

    property string textColor: outgoing ? "white" : "#333333"

    property bool sectionVisible: isNewDay || isAction
    property string sectionText: {
        if (isAction) {
            return delegateUtils.getActionMessageText(
                        outgoing, senderName, actionType,
                        actionUser, actionTitle, actionThumbnail);
        } else if (isNewDay) {
            return newDayText;
        } else return "";
    }

    anchors {
        left: parent ? parent.left : undefined
        right: parent ? parent.right : undefined
    }
    height: isAction || isNewDay ? section.height + internalDelegate.height + units.gu(2)
                                 : internalDelegate.height + units.gu(1)

    signal clicked()
    signal profileImageClicked()

    color: Qt.rgba(0, 0, 0, 0)

    Column {
        id: column
        anchors {
            left: parent.left
            right: parent.right
            verticalCenter: parent.verticalCenter
        }
        spacing: units.gu(1)

        TelegramSection {
            id: section
            text: sectionText
            image: actionThumbnail
            visible: sectionVisible
        }

        Item {
            id: internalDelegate
            anchors {
                left: parent ? parent.left : undefined
                right: parent ? parent.right : undefined
            }
            height: isAction ? 0 : bubble.height
            visible: !isAction

            UbuntuShape {
                id: imageShape
                anchors {
                    left: parent.left
                    leftMargin: image.source == "" ? 0 : units.gu(1)
                    bottom: sectionVisible ? bubble.bottom : parent.bottom
                }

                width: image.source == "" || image.source === undefined ? 0 : units.gu(5);
                height: width

                image: Image {
                    id: image
                    anchors.fill: parent

                    asynchronous: true
                    fillMode: Image.PreserveAspectCrop
                    source: senderImage
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: messageDelegate.profileImageClicked()
                }
            }

            TelegramBubble {
                id: bubble
                outgoing: messageDelegate.outgoing
                anchors {
                    top: parent.top
                    left: outgoing ? undefined: imageShape.right
                    leftMargin: units.gu(2)
                    right: outgoing ? parent.right : undefined
                    rightMargin: units.gu(2)
                }
                height: messageContents.height + units.gu(1)
                width: Math.max(messageLabel.width, senderLabel.width, messageStatusRow.width, thumbnailImage.width)
                       + (hasThumbnail ? units.gu(1) : units.gu(2))

                Item {
                    id: messageContents
                    anchors {
                        top: parent.top
                        topMargin: units.gu(0.5)
                        left: parent.left
                        leftMargin: hasThumbnail ? units.gu(0.5) : units.gu(1)
                        right: parent.right
                        rightMargin: hasThumbnail ? units.gu(0.5) : units.gu(1)
                    }
                    height: childrenRect.height

                    Text {
                        id: senderLabel
                        anchors.top: parent.top
                        height: text === "" ? 0 : implicitHeight
                        font.pixelSize: FontUtils.sizeToPixels("small")
                        font.weight: Font.Normal
                        color: senderColor
                        visible: !isPhotoOrVideo
                        text: isPhotoOrVideo ? "" : senderDisplayName
                    }

                    Label {
                        id: messageLabel
                        anchors{
                            top: senderLabel.bottom
                        }
                        height: paintedHeight
                        width: Math.min(implicitWidth, 0.7 * internalDelegate.width)
                        wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                        fontSize: mediaType != TLMessageMedia.TypeMessageMediaEmpty ?
                                "small" : userSettings.contents.fontSize
                        color: textColor
                        text: parseText(message)
                        textFormat: Text.RichText
                        onLinkActivated: Qt.openUrlExternally(link)

                        // Taken from messaging-app
                        function parseText(text) {
                            var phoneExp = /(\+?([0-9]+[ ]?)?\(?([0-9]+)\)?[-. ]?([0-9]+)[-. ]?([0-9]+)[-. ]?([0-9]+))/img;
                            // remove html tags
                            text = text.replace(/</g,'&lt;').replace(/>/g,'<tt>&gt;</tt>');
                            // replace line breaks
                            text = text.replace(/(\n)+/g, '<br />');
                            // check for links
                            var htmlText = BaLinkify.linkify(text);
                            if (htmlText !== text) {
                                return htmlText
                            }
                            // linkify phone numbers if no web links were found
                            return text.replace(phoneExp, '<a href="tel:///$1">$1</a>');
                        }
                    }

                    Item {
                        id: thumbnailBox

                        anchors {
                            horizontalCenter: parent.horizontalCenter
                            top: senderLabel.bottom
                        }
                        width: {
                            if (isPhotoOrVideo) {
                                if (thumbnailImage.source == "") {
                                    height = units.gu(12);
                                    return units.gu(12);
                                } else {
                                    if (thumbnailImage.isLandscape) {
                                        height = units.gu(14);
                                        return units.gu(26);
                                    } else {
                                        height = units.gu(26);
                                        return units.gu(14);
                                    }
                                }
                            } else {
                                height = 0;
                                return 0;
                            }
                        }

                        visible: isPhotoOrVideo

                        Image {
                            id: thumbnailImage

                            property bool isLandscape: false
                            property real aspectRatio: 1

                            anchors.fill: parent
                            asynchronous: true
                            fillMode: Image.PreserveAspectCrop

                            sourceSize.width: messageDelegate.width
                            sourceSize.height: 0 // scale accordingly
                            source: Qt.resolvedUrl(thumbnail)

                            onStatusChanged:  {
                                if (status === Image.Error) {
                                    width = 128
                                    height = 128
                                    source = "image://theme/image-missing"
                                    console.warn("Image error, source is: _" + source + "_");
                                } else if (status === Image.Ready) {
                                    isLandscape = implicitWidth > implicitHeight
                                    aspectRatio = implicitWidth / implicitHeight
                                }
                            }
                        }

                        FastBlur {
                            anchors.fill: thumbnailImage
                            transparentBorder: true
                            radius: isVideo ? 32 : 0 // Workaround for: enabled
                            // enabled: isVideo // This doesn't work!
                            source: thumbnailImage
                        }

                        Rectangle {
                            id: videoIndicator
                            anchors {
                                top: thumbnailImage.top
                                topMargin: units.gu(0.5)
                                left: thumbnailImage.left
                                leftMargin: units.gu(0.5)
                            }
                            width: childrenRect.width + units.gu(1)
                            height: units.gu(2)
                            visible: isVideo

                            color: "#64000000"
                            radius: 2

                            Image {
                                anchors {
                                    left: parent.left
                                    leftMargin: units.gu(0.5)
                                    verticalCenter: parent.verticalCenter
                                }
                                height: units.gu(1.5)
                                fillMode: Image.PreserveAspectFit
                                smooth: true
                                antialiasing: true
                                asynchronous: true
                                source: Qt.resolvedUrl("../images/ic_video.png")
                            }
                        }

                        Image {
                            id: thumbnailImageOverlay
                            anchors.centerIn: thumbnailImage
                            width: units.gu(6)
                            height: width
                            visible: isVideo || (isPhoto && needsDownload)

                            source: {
                                if (needsDownload) {
                                    if (isDownloading) {
                                        return Qt.resolvedUrl("../images/photocancel.png");
                                    } else {
                                        return Qt.resolvedUrl("../images/photoload.png");
                                    }
                                } else {
                                    return Qt.resolvedUrl("../images/playvideo.png");
                                }
                            }
                        }

                        MouseArea {
                            anchors.fill: thumbnailImage
                            onClicked: {
                                if (isDownloading) {
                                    cancelDownload();
                                } else if (needsDownload) {
                                    isVideo ? downloadVideo() : downloadPhoto();
                                } else {
                                    openPreview();
                                }
                            }

                            function downloadVideo() {
                                telegramClient.downloadVideo(messageId);
                            }

                            function downloadPhoto() {
                                telegramClient.downloadPhoto(messageId);
                            }

                            function cancelDownload() {
                                telegramClient.cancelDownload(messageId);
                            }

                            function openPreview() {
                                var properties;
                                if (mediaType === TLMessageMedia.TypeMessageMediaPhoto) {
                                    var preview = photo !== "" ? photo : thumbnail
                                    properties = { "senderName": senderName, "photoPreviewSource": preview };
                                } else if (mediaType === TLMessageMedia.TypeMessageMediaVideo) {
                                    properties = { "senderName": senderName, "videoPreviewSource": video };
                                }
                                pageStack.push(previewPage, properties);
                            }
                        }

                        ProgressBar {
                            anchors {
                                left: thumbnailImage.left
                                bottom: thumbnailImage.bottom
                                right: thumbnailImage.right
                            }
                            height: units.dp(3)
                            showProgressPercentage: false
                            minimumValue: 0
                            maximumValue: 100
                            visible: isDownloading
                            value: progress
                        }
                    }

                    Row {
                        id: messageStatusRow
                        anchors {
                            top: isPhotoOrVideo ? undefined : messageLabel.bottom
                            right: parent.right
                            rightMargin: units.dp(2)
                            bottom: isPhotoOrVideo ? thumbnailBox.bottom : undefined
                            bottomMargin: units.dp(4)
                        }
                        spacing: units.dp(4)

                        Label {
                            id: timeLabel
                            font.weight: Font.DemiBold
                            fontSize: "x-small"
                            color: {
                                if (isPhotoOrVideo && hasThumbnail) {
                                    return Color.white;
                                }
                                return outgoing ? Color.dark_green : Color.grey;
                            }
                            text: time
                        }

                        Image {
                            id: messageSentStatus
                            anchors {
                                verticalCenter: timeLabel.verticalCenter
                                right: time.left
                                rightMargin: units.dp(4)
                            }
                            width: units.gu(2)
                            height: width
                            visible: outgoing
                            z: 1
                            fillMode: Image.PreserveAspectFit
                            source: {
                                if (!isSent) {
                                    return Qt.resolvedUrl("../images/msg_clock_white.png");
                                } else if (!isRead) {
                                    return Qt.resolvedUrl("../images/Checks1_2x_white.png");
                                } else {
                                    return Qt.resolvedUrl("../images/Checks2_2x_white.png");
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
