import QtQuick 2.0

// This component is intented for use in Ubuntu Header content.
Item {
    property alias title: titleText.text
    property alias subtitle: subtitleText.text
    property alias image: headerImage.source

    property bool isConnecting: false

    id: header
    height: parent.height

    Image {
        id: headerImage
        width: height
        height: parent.height/2
        anchors {
            left: parent.left
            verticalCenter: parent.verticalCenter
        }
        source: Qt.resolvedUrl("../images/ic_ab_logo.png")
        RotationAnimation {
            id: connectingAnimation
            target: headerImage
            direction: RotationAnimation.Clockwise
            from: 0
            to: 359
            loops: Animation.Infinite
            duration: 5000
            alwaysRunToEnd: false
            running: isConnecting

            onRunningChanged: {
                if (!running) {
                    connectingAnimation.stop();
                    headerImage.rotation = 0;
                }
            }
        }
    }

    TelegramLabel {
        id: titleText
        // We need fixed width. Otherwise, we overflow action icons.
        width: Math.min(implicitWidth, parent.width - headerImage.width - anchors.leftMargin)
        anchors {
            top: parent.top
            topMargin: units.gu(1)
            left: headerImage.right
            leftMargin: units.gu(1)

            verticalCenter: subtitleText.visible ? undefined : parent.verticalCenter
        }
        verticalAlignment: Text.AlignVCenter
        height: subtitleText.visible ? parent.height / 2 : implicitHeight

        font.pixelSize: FontUtils.sizeToPixels("large")
        elide: Text.ElideRight
        text: isConnecting ? i18n.tr("Connecting...") : i18n.tr("Telegram")
    }

    Text {
        id: subtitleText
        width: Math.min(implicitWidth, parent.width - headerImage.width - anchors.leftMargin)
        anchors {
            left: headerImage.right
            leftMargin: units.gu(1)
            bottom: parent.bottom
            bottomMargin: units.gu(0.5)
        }
        verticalAlignment: Text.AlignVCenter
        height: parent.height / 2
        visible: text.length > 0

        font.pixelSize: FontUtils.sizeToPixels("small")
        elide: Text.ElideRight
    }
}
