import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Connectivity 1.0
import "TelegramColors.js" as TelegramColors

Page {
    id: page

    property bool isSearching: false
    property bool isOnline: NetworkingStatus.online
    property bool isConnected: {
        if (onlineIndicationOnly) {
            return isOnline;
        } else if (root.activeFocus) {
            return telegramClient.connected;
        } return true; // Don't indicate connectivity.
    }

    property bool onlineIndicationOnly: false

    property alias title: header.title
    property alias subtitle: header.subtitle
    property alias body: body.children
    property alias searchTerm: searchField.text
    property alias busy: activityIndicator.running

    TelegramHeader {
        id: header
        isConnecting: !page.isConnected
        visible: !isSearching
        width: parent ? parent.width - units.gu(2) : undefined
    }

    TextField {
        id: searchField
        visible: isSearching
        width: header.width
        inputMethodHints: Qt.ImhNoPredictiveText

        onTextChanged: {
            if (typeof onSearchTermChanged === 'function') {
                onSearchTermChanged(text);
            }
        }
    }

    head {
        id: head
        contents: isSearching ? searchField : header

        backAction: Action {
            id: backAction
            iconName: "back"
            onTriggered: {
                if (isSearching) {
                    searchFinished();
                    return;
                }
                pageStack.pop();
                if (typeof onBackPressed === 'function') {
                    onBackPressed();
                }
            }
        }
    }

    Rectangle {
        id: body
        anchors.fill: parent
        // Due to some fancy Page behavior, in fact,
        // this doesn't end up as white anyway..
        color: TelegramColors.page_background
    }

    ActivityIndicator {
        id: activityIndicator
        anchors.centerIn: parent
        z: 101
    }

    Rectangle {
        id: activityBackground
        anchors.fill: parent
        z: 100
        color: "#44000000"
        visible: activityIndicator.running
        MouseArea {
            anchors.fill: parent
            preventStealing: true
            enabled: activityIndicator.running
        }
    }

    function searchPressed() {
        isSearching = true;
        searchField.forceActiveFocus();
    }

    function searchFinished() {
        if (!isSearching) return;

        isSearching = false;
        searchField.text = "";
    }
}
