====================
Some debugging notes
====================

This document is still a work in progress.

Here are some very rough notes on how to set up Telegram for debugging.

Build the library in debug mode
===============================
Append to the cmake line in build.sh:

    ${SDK_PATH}/qtc_chroot_cmake ${CLICK_SDK_ARCH} ${CLICK_SDK_FRAMEWORK} $SERIES ../.. -DCMAKE_BUILD_TYPE=Debug


gdb + debug symbols
===================
When stepping through a stack trace in gdb, it may be useful to have debug
symbols for the Qt libraries. 

In Ubuntu, the debug symbol packages  are built and hosted on a different
server from the standard source and binary packages. To get them:

    echo "deb http://ddebs.ubuntu.com $(lsb_release -cs) main restricted universe multiverse" | sudo tee -a /etc/apt/sources.list.d/ddebs.list
    sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys 428D7C01

And then of course, issue ``sudo apt-get update``.

The next step is to install the symbol packages. The package names correspond
to the binary package names, with "-dbgsym" appended. For example, to get
the symbols for qtbase5-dev:

    sudo apt-get install qtbase5-dev-dbgsym

Beware that the symbols can be quite large in terms of disk space, and it is
assumed you are debugging with gdb on your desktop, where storage is less of
an issue than a device.

Finally, understand that installing debug symbols only partially improves
the debugging experience in gdb. An additional nicety is to install the
actual library source, so that you can easily view source code from gdb
itself.

Here is one technique to do so. First, download the actual source for the
package you are interested in:

    mkdir debug
    cd debug
    apt-get source qtbase5-dev
    cd ..

Next, you must start gdb properly so it knows how to find the new source
files. The key rune below is: ``find debug -type d -printf '-d %p'``

    LD_LIBRARY_PATH=$LD_LIBRARY_PATH:build/amd64/library/lib QML2_IMPORT_PATH=build/amd64/library/qmlplugin gdb qmlscene `find debug -type d -printf '-d %p '`

After starting gdb, just a few extra commands, and you are on your
way:

    set args telegram.qml
    r

Now, when you get a crash and inspect the stack trace, you can view the
source:

    (gdb) bt
    #0  0x00007ffff5232117 in __GI_raise (sig=sig@entry=6)
        at ../nptl/sysdeps/unix/sysv/linux/raise.c:56
    #1  0x00007ffff5233808 in __GI_abort () at abort.c:89
    #2  0x00007ffff5273fd4 in __libc_message (do_abort=do_abort@entry=1, 
        fmt=fmt@entry=0x7ffff537dba8 "*** Error in `%s': %s: 0x%s ***\n")
        at ../sysdeps/posix/libc_fatal.c:175
    #3  0x00007ffff527b5e6 in malloc_printerr (ptr=<optimized out>, 
        str=0x7ffff537dc90 "double free or corruption (!prev)", action=1)
        at malloc.c:4996
    #4  _int_free (av=<optimized out>, p=<optimized out>, have_lock=0)
        at malloc.c:3840
    #5  0x00007ffff5ec9608 in QObject::event (this=0xc32780, e=<optimized out>)
        at kernel/qobject.cpp:1232

    (gdb) frame 5
    #5  0x00007ffff5ec9608 in QObject::event (this=0xc32780, e=<optimized out>)
            at kernel/qobject.cpp:1232
    1232	        qDeleteInEventHandler(this);
    (gdb) l
    1227	    case QEvent::ChildRemoved:
    1228	        childEvent((QChildEvent*)e);
    1229	        break;
    1230	
    1231	    case QEvent::DeferredDelete:
    1232	        qDeleteInEventHandler(this);
    1233	        break;
    1234	
    1235	    case QEvent::MetaCall:
    1236	        {


Running with valgrind
=====================
Valgrind is a suite of tools, one of which is memcheck, which can be used to
detect memory leaks. Running it can be a bit complex, but this rune should
help:

    LD_LIBRARY_PATH=$LD_LIBRARY_PATH:build/amd64/library/lib QML2_IMPORT_PATH=build/amd64/library/qmlplugin G_SLICE=always-malloc G_DEBUG=gc-friendly valgrind -v --tool=memcheck --leak-check=full --num-callers=40 --log-file=valgrind.log --track-origins=yes --trace-children=yes qmlscene telegram.qml

If you find the log to be noisy, you may wish to suppress some of the
warnings. Google for "valgrind suppressions" for the details. In the above
rune, you can add ``--gen-suppressions=all`` to generate suppressions.

Check out the ``tools/extract_suppressions.sh`` script and go from there.
