#include"preview.h"

#include <unity/scopes/PreviewWidget.h>
#include <unity/scopes/ColumnLayout.h>
#include <unity/scopes/PreviewReply.h>
#include <unity/scopes/VariantBuilder.h>

#include <QString>
#include <QDebug>

using namespace unity::scopes;
using namespace std;

TelegramPreview::TelegramPreview(Result const& result, ActionMetadata const& metadata) :
    PreviewQueryBase(result, metadata)
{
    setlocale(LC_ALL, "");
    textdomain(GETTEXT_DOMAIN.toStdString().c_str());

}

TelegramPreview::~TelegramPreview()
{
}

void TelegramPreview::cancelled()
{
}

void TelegramPreview::run(unity::scopes::PreviewReplyProxy const& reply)
{
    QString telegram_uri ="telegram://";
    ColumnLayout layout1col(1);
    layout1col.add_column({"header", "art","description", "actions"});

    ColumnLayout layout2col(2);
    layout2col.add_column({"header","art"});
    layout2col.add_column({"description", "actions"});

    ColumnLayout layout3col(3);
    layout3col.add_column({"header"});
    layout3col.add_column({"description", "actions"});
    layout3col.add_column({"art"});
    reply->register_layout({layout1col, layout2col, layout3col});

    auto myresult = result();

    PreviewWidget header("header", "header");
    if(myresult.contains("contact")) {

        header.add_attribute_mapping("title", "title");
        header.add_attribute_mapping("subtitle", "phone");
        header.add_attribute_mapping("mascot", "art");
        reply->push({header});

        string mess_uri = telegram_uri.toStdString() + "chat/"+ myresult["id"].get_string();
        string call_uri = "tel:///" + myresult["phone"].get_string();

        PreviewWidget actions("actions", "actions");
        {
            VariantBuilder builder;
            builder.add_tuple({
                                  {"id", Variant("Open")},
                                  //Translator: this is open , as in "open the tasks app"
                                  {"label", Variant(_("Open chat"))},
                                  {"uri", Variant(mess_uri)}
                              });

        if (!myresult["phone"].get_string().empty()) {
            builder.add_tuple({
                                  {"id", Variant("call")},
                                  //TRANSLATOR - this is call as  in make a phone call
                                  {"label", Variant(_("Call"))},
                                  {"uri", Variant(call_uri)}
                              });
        }
        actions.add_attribute_value("actions", builder.end());
        }
        reply->push({actions});

        return;
    }
    if(myresult.contains("text")) {
        header.add_attribute_mapping("title", "from");
        header.add_attribute_mapping("subtitle", "date");
        header.add_attribute_mapping("mascot", "art");

        PreviewWidget description("description", "text");
        description.add_attribute_mapping("text", "title");
        reply->push({header,description});

        string mess_uri = telegram_uri.toStdString() + "chat/"+ myresult["groupchat"].get_string();

        PreviewWidget actions("actions", "actions");
        {
            VariantBuilder builder;
            builder.add_tuple({
                                  {"id", Variant("Open")},
                                  //Translator: this is open , as in "open the tasks app"
                                  {"label", Variant(_("Open chat"))},
                                  {"uri", Variant(mess_uri)}
                              });
           actions.add_attribute_value("actions", builder.end());
        }
        reply->push({actions});
        return;
    }
    if(myresult.contains("photo")) {
        header.add_attribute_mapping("title", "from");
        header.add_attribute_mapping("subtitle", "date");
        if(myresult.contains("attachment"))
            header.add_attribute_mapping("mascot", "contactpic");
        else
            header.add_attribute_mapping("mascot", "art");

        PreviewWidget description("description", "text");
        description.add_attribute_mapping("text", "text");
        reply->push({header,description});

        PreviewWidget art("art","image");
        art.add_attribute_mapping("source", "mediaUrl");
        reply->push({art});

        string mess_uri = telegram_uri.toStdString() + "chat/"+ myresult["groupchat"].get_string();

        PreviewWidget actions("actions", "actions");
        {
            VariantBuilder builder;
            builder.add_tuple({
                                  {"id", Variant("Open")},
                                  //Translator: this is open , as in "open the tasks app"
                                  {"label", Variant(_("Open chat"))},
                                  {"uri", Variant(mess_uri)}
                              });
           actions.add_attribute_value("actions", builder.end());
        }
        reply->push({actions});
        return;
    }

    if(myresult.contains("video")) {
        header.add_attribute_mapping("title", "from");
        header.add_attribute_mapping("subtitle", "date");
        if(myresult.contains("attachment")) {
            header.add_attribute_mapping("mascot", "contactpic");
        } else {
            header.add_attribute_mapping("mascot", "art");
        }
        reply->push({header});

        auto videoUrl = QString::fromStdString(myresult["videoUrl"].get_string());

        if (videoUrl.isEmpty()){
            PreviewWidget description("description", "text");
            description.add_attribute_value("text", Variant(_("Video has not been downloaded from server, please do this from the app")));
            reply->push({description});
            PreviewWidget art("art","image");
            art.add_attribute_mapping("source", "mediaUrl");
            reply->push({art});

        } else {
            videoUrl = "video://" + videoUrl;
            PreviewWidget art("art","video");
            art.add_attribute_value("source", Variant(videoUrl.toStdString()));
            art.add_attribute_mapping("screenshot", "mediaUrl");

            reply->push({art});
        }

        string mess_uri = telegram_uri.toStdString() + "chat/" + myresult["groupchat"].get_string();

        PreviewWidget actions("actions", "actions");
        {
            VariantBuilder builder;
            builder.add_tuple({
                                  {"id", Variant("Open")},
                                  //Translator: this is open , as in "open the tasks app"
                                  {"label", Variant(_("Open chat"))},
                                  {"uri", Variant(mess_uri)}
                              });
           actions.add_attribute_value("actions", builder.end());
        }
        reply->push({actions});
        return;
    }
    header.add_attribute_mapping("title", "from");
    header.add_attribute_mapping("subtitle", "date");
    header.add_attribute_mapping("mascot", "art");

    PreviewWidget description("description", "text");
    description.add_attribute_mapping("text", "text");
    reply->push({header,description});

    string mess_uri = telegram_uri.toStdString() + "chat/" + myresult["groupchat"].get_string();
    PreviewWidget actions("actions", "actions");
    {
        VariantBuilder builder;
        builder.add_tuple({
                              {"id", Variant("Open")},
                              //Translator: this is open , as in "open the tasks app"
                              {"label", Variant(_("Open chat"))},
                              {"uri", Variant(mess_uri)}
                          });
        actions.add_attribute_value("actions", builder.end());
    }
    reply->push({actions});
    return;
}

