#include "query.h"
#include <unity/scopes/Annotation.h>
#include <unity/scopes/CategoryRenderer.h>
#include <unity/scopes/QueryBase.h>
#include <unity/scopes/SearchReply.h>

#include <QDebug>
#include <QUrl>
#include <QDateTime>
#include <set>

#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>


using namespace unity::scopes;
using namespace std;

namespace
{
const static std::string UNREAD_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "grid",
        "card-layout": "horizontal",
        "card-size": "small",
        "collapsed-rows": 1
},
        "components": {
        "title": "title",
        "mascot" : "art",
        "subtitle": "subtitle"
}
}
        )";
}

const static std::string READ_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "grid",
        "card-layout": "horizontal",
        "card-size": "small",
        "collapsed-rows": 2
},
        "components": {
        "title": "title",
        "mascot" : "art",
        "subtitle": "subtitle"
}
}
        )";


const static std::string ERROR_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "grid",
        "card-layout": "horizontal",
        "card-size": "small"
},
        "components": {
        "title": "title",
        "subtitle": "subtitle"
}
}
)";

const static std::string PHOTO_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "grid",
        "card-size": "small",
        "collapsed-rows": 0
},
        "components": {
        "art" : "art"
}
}
        )";

const static std::string CARR_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "carousel",
        "card-size": "small",
        "overlay": true
},
        "components": {
        "title": "title",
        "art" : "art"
}
}
        )";
const static std::string CONTACTS_SEARCH_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "grid",
        "card-layout": "horizontal",
        "card-size": "small",
        "collapsed-rows": 5
},
        "components": {
        "title": "title",
        "art" : "art"
}
}
        )";

const static std::string SEARCH_MESS_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "grid",
        "card-layout": "horizontal",
        "card-size": "small",
        "collapsed-rows": 5
},
        "components": {
        "title": "title",
        "mascot" : "art",
        "subtitle": "subtitle"
}
}
        )";



void errorResult(unity::scopes::SearchReplyProxy const& reply){
     CategoryRenderer catErrorRendr(ERROR_TEMPLATE);
     auto errorMessages = reply->register_category("error", _(""), "", catErrorRendr);
     CategorisedResult res(errorMessages);
     res.set_uri("http:/ubuntu.com");
     res["title"] = _("Could not retrieve your telegram data");
     res["subtitle"] = _("Please make sure you have a valid account set up");
     reply->push(res);
}

TelegramQuery::TelegramQuery(CannedQuery const& query, SearchMetadata const& metadata, QString const& scopeDir) :
    SearchQueryBase(query, metadata), mScopeDir(scopeDir)
{
 setlocale(LC_ALL, "");
 textdomain(GETTEXT_DOMAIN.toStdString().c_str());

}

TelegramQuery::~TelegramQuery()
{
}

void TelegramQuery::cancelled()
{
}

void TelegramQuery::getOurId(QString phoneNumber){
    QString ourIdField = "ourId=";
    QString authFilePath = "auth";
    QString fileName = U1DBPath + phoneNumber + "/" + authFilePath;

    auto authFile = new QFile (fileName);
    if (!authFile->open(QIODevice::ReadOnly)){
        qCritical()<< "opening file failed" << fileName;
        mOurId = 0;
        return;
    }

    QTextStream streamText(authFile);
    QString authContent = streamText.readAll();

    int position = authContent.indexOf(ourIdField);
    if (position < 0){
        qCritical()<< "could not find" << ourIdField;
        mOurId = 0;
        return;
    }
    int positionEnd = authContent.indexOf( "\n" , position);
    if (positionEnd < 0){
        mOurId = 0;
        return;
    }
    position = position + ourIdField.size();
    mOurId = authContent.mid(position, positionEnd - position).trimmed().toInt();
}

bool TelegramQuery::openDb() {
    QString dbPath = DatabasePath;
    QString dbsuffix = "*.sqlite";
    QString phoneNumberPrefix = "+*";

    QDir dbDir(U1DBPath);

    if (!dbDir.exists()) {
        qCritical() << "path not found " <<U1DBPath;
        return false;
    }
    QStringList filters;
    filters << u1dbname;
    dbDir.setNameFilters(filters);

    auto iresults = dbDir.entryInfoList();
    if(iresults.size()<1){
        qCritical() << "no database found in " <<U1DBPath;
        return false;
    }
    auto u1dbFile = iresults[0].absoluteFilePath();
    QSqlDatabase u1DB = QSqlDatabase::addDatabase("QSQLITE");
    u1DB.setDatabaseName(u1dbFile);

    if (!u1DB.open()) {
        qCritical() << "u1db failed to open ";
        return false;
    }
    QSqlQuery settings(u1DB);
    settings.prepare(U1DB_SETTINGS);
    if (!settings.exec()) {
        qCritical() << "faile to execute u1db querry";
        return false;
    }

    int fieldContent = settings.record().indexOf(U1DB_content);
    if (fieldContent < 0) {
        qCritical() << "faile to get record from U1DB";
        return false;
    }
    if (!settings.first()) {
        qCritical() << "faile to get entry from U1DB";
        return false;
    }
    QByteArray resultString =settings.value(fieldContent).toString().toStdString().c_str();
    QJsonParseError e;
    QJsonDocument document = QJsonDocument::fromJson(resultString, &e);
    if (e.error != QJsonParseError::NoError) {
        qCritical() << "failed to parse response " << e.errorString() << "\n" << QString(resultString);
        throw QString("Failed to parse response: %1").arg(e.errorString());
    }
    QJsonObject object = document.object();
    QString phoneNumber = object.value("phoneNumber").toString().replace(" ","");

    //fetch our id for the found number
    getOurId(phoneNumber);

    QString dbNumPath = dbPath + phoneNumber;
    QDir dbDirNum(dbNumPath);

    filters.clear();
    filters << dbsuffix;
    dbDirNum.setNameFilters(filters);

    auto iresultsDBs = dbDirNum.entryInfoList();
    if(iresultsDBs.size()<1){
        qCritical() << "no phone number found in path " <<dbNumPath;
        return false;
    }
    auto dbFile = iresultsDBs[0].absoluteFilePath();

    //ready to open db
    mDb = QSqlDatabase::addDatabase("QSQLITE");
    mDb.setDatabaseName(dbFile);
    return mDb.open();
}

int TelegramQuery::getUnreadMessages(std::vector<TelegramMessage>  & unreadMessages){
    if (!mDb.isOpen()) {
        if (!openDb())
            return 0;
    }
    QSqlQuery db_results(mDb);
    db_results.prepare(DIALOGS_QUERY.arg(GET_ALL));
    if (!db_results.exec()){
        return 0;
    }
    int fieldUnread = db_results.record().indexOf(DIALOG_UNREAD);
    int fieldId = db_results.record().indexOf(ID_);
    int fieldIsChat = db_results.record().indexOf(DIALOG_ISCHAT);

    int totalunread = 0;
    while (db_results.next()) {
        auto unread = db_results.value(fieldUnread).toInt();
        totalunread+= unread;
        if (unread > 0) {
            QString chatid = db_results.value(fieldId).toString();
            std::vector<TelegramMessage> iresults;
            if(db_results.value(fieldIsChat).toBool()) {
                iresults = getMessagesFromDB(MSG_QUERY.arg(GET_TO_MSG_QUERY.arg(chatid).arg(unread)));
            } else {
                iresults = getMessagesFromDB(MSG_QUERY.arg(GET_FROM_MSG_QUERY.arg(chatid).arg(unread)));
            }
            for (unsigned int i = 0 ; i<iresults.size(); i++)
                unreadMessages.push_back(iresults[i]);
        }
    }
    return totalunread;
}

std::vector<TelegramMessage> TelegramQuery::getMessagesFromDB(QString querytext){
    std::vector<TelegramMessage> results;
    if (!mDb.isOpen()) {
        if (!openDb())
            return results;
    }
    QSqlQuery db_results(mDb);
    db_results.prepare(querytext);
    if (!db_results.exec()) {
        return results;
    }
    int fieldText = db_results.record().indexOf(MESSAGE_TEXT);
    int fieldPhone = db_results.record().indexOf(MESSAGE_Phone);
    int fieldThumbnail = db_results.record().indexOf(MESSAGE_Thumbnail);
    int fieldFN = db_results.record().indexOf(MESSAGE_firstName);
    int fieldSN = db_results.record().indexOf(MESSAGE_lastName);
    int fieldUNRead = db_results.record().indexOf(MESSAGE_unread);
    int fieldDate = db_results.record().indexOf(MESSAGE_date);
    int fieldOut = db_results.record().indexOf(MESSAGE_out);
    int fieldMediaType = db_results.record().indexOf(MESSAGE_mediatype);
    int fieldFromId = db_results.record().indexOf(MESSAGE_fromid);
    int fieldMsgId = db_results.record().indexOf(ID_);
    int fieldVideo = db_results.record().indexOf(MESSAGE_VIDEO);
    int fieldPhoto = db_results.record().indexOf(MESSAGE_PHOTO);
    int fieldIsChat = db_results.record().indexOf(MESSAGE_ISCHAT);
    int fieldToID = db_results.record().indexOf(MESSAGE_TOID);
    int fieldPhotoLocal = db_results.record().indexOf(MESSAGE_PHOTOLOCAL);

    while (db_results.next()) {
        TelegramMessage myMessage;
        myMessage.text = db_results.value(fieldText).toString();
        myMessage.from.phone = db_results.value(fieldPhone).toString();
        myMessage.from.thumbNail = db_results.value(fieldThumbnail).toString();
        myMessage.from.firstName = db_results.value(fieldFN).toString();
        myMessage.from.lastName = db_results.value(fieldSN).toString();
        myMessage.from.id = db_results.value(fieldFromId).toInt();
        myMessage.out = db_results.value(fieldOut).toBool();
        myMessage.unread = db_results.value(fieldUNRead).toBool();
        myMessage.date = (qint64) db_results.value(fieldDate).toInt();
        myMessage.mediaType = db_results.value(fieldMediaType).toInt();
        myMessage.id = db_results.value(fieldMsgId).toInt();
        auto urlVideo = db_results.value(fieldVideo).toString();
        auto urlPhoto = db_results.value(fieldPhoto).toString();
        myMessage.mediaUrl = urlVideo.isEmpty()?urlPhoto:urlVideo;
        myMessage.from.lastName = db_results.value(fieldSN).toString();
        myMessage.groupChat =  db_results.value(fieldIsChat).toBool();
        myMessage.groupChatId =db_results.value(fieldToID).toString();
        myMessage.videoThumbnail = db_results.value(fieldPhotoLocal).toString();

        results.push_back(myMessage);
    }
    return results;
}
QString getDate(qint64 mytime){
    QDateTime mydate= QDateTime::fromMSecsSinceEpoch(mytime);
    return mydate.toString(TIMEFORMAT);
}

QString getImageUrl(int id, QString scopePath){
    QString images[] = {
       "_violet.png",
        "_pink.png" ,
        "_green.png",
        "_yellow.png",
        "_blue.png"  ,
        "_aqua.png"  ,
        "_orange.png",
        "_red.png"
    };
    QString path = "file://"+scopePath+"/images/user"+images[id % 8];
    return path;
}

CategorisedResult TelegramQuery::contactToResult (Category::SCPtr category, TelegramContact contact){
    CategorisedResult res(category);
    res.set_uri("http:/ubuntu.com");
    res["title"] = (contact.firstName +" " +contact.lastName).toStdString();
    res["art"]= contact.thumbNail.isEmpty()?getImageUrl(contact.id,mScopeDir).toStdString():contact.thumbNail.toStdString();
    res["contact"] = "yes";
    res["phone"] = contact.phone.isEmpty()?"":("+" + contact.phone).toStdString();
    res["id"] = QString::number(contact.id).toStdString();
    return res;
}

CategorisedResult TelegramQuery::messageToResult (Category::SCPtr category, TelegramMessage message, bool useAttachment){
    CategorisedResult res(category);
    res.set_uri("http:/ubuntu.com");
    switch (message.mediaType) {
    case MessageMediaType::typeMessageMediaEmpty:
        res["title"] = message.text.toStdString();
        res["text"] = "yes";
        break;
    case MessageMediaType::typeMessageMediaAudio:
        res["title"] = _("Audio message");
        res["audio"] = "yes";
        break;
    case MessageMediaType::typeMessageMediaContact:
        res["title"] = _("Contact received");
        res["vcard"] = "yes";
        break;
    case MessageMediaType::typeMessageMediaDocument:
        res["title"] = _("Document received");
        res["doc"] = "yes";
        break;
    case MessageMediaType::typeMessageMediaPhoto:
        res["title"] = _("Picture message");
        res["photo"] = "yes";
        res["mediaUrl"]= message.mediaUrl.toStdString();
        break;
    case MessageMediaType::typeMessageMediaVideo:
        res["title"] = _("Video message");
        res["video"] = "yes";
        res["videoUrl"] = message.mediaUrl.toStdString();
        res["mediaUrl"] = message.videoThumbnail.toStdString();
        break;
    default:
        res["title"] = _("Special message type");
    }
    res["subtitle"] = (message.from.firstName + " " + message.from.lastName + " @ " + getDate(message.date*1000)).toStdString();
    res["from"] = (message.from.firstName + " " + message.from.lastName).toStdString();
    res["date"] = getDate(message.date*1000).toStdString();

    if (useAttachment) {
        if (res.contains("video"))
            res["art"] = message.videoThumbnail.toStdString();
        else
            res["art"] = message.mediaUrl.toStdString();
        res["contactpic"] = message.from.thumbNail.isEmpty()?getImageUrl(message.from.id,mScopeDir).toStdString():message.from.thumbNail.toStdString();
        res["attachment"] = true;
    }else
       res["art"] = message.from.thumbNail.isEmpty()?getImageUrl(message.from.id,mScopeDir).toStdString():message.from.thumbNail.toStdString();

    res["id"] = QString::number(message.id).toStdString();
    if (message.groupChat) {
        res["groupchat"] = message.groupChatId.toStdString();
    } else {
        res["groupchat"] = QString::number(message.from.id).toStdString();
    }
    return res;
}

void TelegramQuery::run(unity::scopes::SearchReplyProxy const& reply)
{
    unsigned int recentMax = 10;
    //Tell gettext where to look for textdomain files
    QString tdir = QString("%1/locale/").arg(mScopeDir);
    QString basedir = QString(bindtextdomain(GETTEXT_DOMAIN.toStdString().c_str(), tdir.toStdString().c_str()));
    QString search_query = QString::fromStdString(query().query_string());

    CategoryRenderer catReadRendr(search_query.isEmpty()?READ_TEMPLATE:SEARCH_MESS_TEMPLATE);
    CategoryRenderer catUnreadRendr(UNREAD_TEMPLATE);
    CategoryRenderer carrRendr(search_query.isEmpty()?CARR_TEMPLATE:CONTACTS_SEARCH_TEMPLATE);
    CategoryRenderer photoRendr(PHOTO_TEMPLATE);

    auto readMessages = reply->register_category("read", _("Recent Messages"), "", catReadRendr);
    auto buddiesCat = reply->register_category("buddies", _("People You Talk To"), "", carrRendr);
    auto photoCat = reply->register_category("photos", _("Received Images"), "", photoRendr);;
    auto searchReadMessages = reply->register_category("searchMSG", _("Results in Messages"), "", catReadRendr);
    auto searchBuddiesCat = reply->register_category("searchBuddies", _("Results in People You Talk To"), "", carrRendr);

    std::map<int,TelegramContact> myBuddies;
    std::vector<CategorisedResult> results;

    //check and openDb
    if(!openDb()) {
        qDebug() << "printing error";
        errorResult(reply);
        return;
    }

    //Unread messages
    std::vector<TelegramMessage> myTelegram;
    std::set<int> unReadIds;

    auto totalunread = getUnreadMessages(myTelegram);

    if (totalunread > 0 && search_query.isEmpty()) {
        QString title_unread = _("Unread Messages: %1");
        title_unread =title_unread.arg(totalunread);
        auto unReadMessages = reply->register_category("unread",title_unread.toStdString(), "", catUnreadRendr);

        for (unsigned int i = 0 ; i < myTelegram.size(); i++){
            unReadIds.insert(myTelegram[i].id);
            auto res = messageToResult(unReadMessages,myTelegram[i],false);
            results.push_back(res);
        }
    }
    //Recent messages
    myTelegram.clear();
    myTelegram = getMessagesFromDB(MSG_QUERY.arg(GET_ALL_MSG));
    unsigned int recieveMsgs = 0;
    for (unsigned int i = 0 ; i < myTelegram.size(); i++){
        if (myTelegram[i].out || unReadIds.find(myTelegram[i].id) != unReadIds.end())
           continue;
        recieveMsgs++;
        myBuddies[myTelegram[i].from.id] = myTelegram[i].from;
        if (recieveMsgs <= recentMax && search_query.isEmpty()){
            auto res = messageToResult(readMessages,myTelegram[i],false);
            results.push_back(res);
        }
        if (!search_query.isEmpty() && myTelegram[i].text.contains(search_query,Qt::CaseInsensitive)){
            auto res = messageToResult(searchReadMessages,myTelegram[i],false);
            results.push_back(res);
        }
        if (search_query.isEmpty() &&
                (myTelegram[i].mediaType == MessageMediaType::typeMessageMediaPhoto ||
                 myTelegram[i].mediaType == MessageMediaType::typeMessageMediaVideo)) {
            auto res = messageToResult(photoCat,myTelegram[i],true);
            results.push_back(res);
        }
    }
    for (auto& mycontact: myBuddies) {
         if ((mycontact.second.firstName.isEmpty() && mycontact.second.lastName.isEmpty()) ||
                 mycontact.second.id == mOurId) {
             continue;
         }
         QString fullName = mycontact.second.firstName + " " +mycontact.second.lastName;
         if(!search_query.isEmpty())  {
             if (fullName.contains(search_query,Qt::CaseInsensitive)) {
                 auto res = contactToResult(searchBuddiesCat,mycontact.second);
                 results.insert(results.begin(),res);
             }
         } else {
             auto res = contactToResult(buddiesCat,mycontact.second);
             results.insert(results.begin(),res);
         }
    }
    for (unsigned int i = 0; i < results.size(); i++)
        reply->push(results[i]);
}
