/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QtGui/QGuiApplication>
#include <QtQuick/QQuickView>
#include <QtQml/QtQml>
#include <QCommandLineParser>
#include <QCommandLineOption>
#include <QDir>
#include <QDebug>
#include <QLibrary>
#include <QString>
#include <QtQml/QQmlDebuggingEnabler>

//static QQmlDebuggingEnabler debuggingEnabler(false);

int main(int argc, char *argv[]) {
    QGuiApplication app(argc, argv);
    QQuickView view;
    view.setResizeMode(QQuickView::SizeRootObjectToView);

    QCommandLineParser parser;
    parser.setSingleDashWordOptionMode(QCommandLineParser::ParseAsLongOptions);
    QCommandLineOption testabilityOption(
            "testability",
            "Load the testability driver.");
    parser.addOption(testabilityOption);
    parser.addPositionalArgument(
            "uri",
            "Uri to allow launching chat window directly, etc.");
    parser.addHelpOption();

    parser.process(app);

    if (parser.isSet(testabilityOption) || getenv("QT_LOAD_TESTABILITY")) {
        QLibrary testLib(QLatin1String("qttestability"));
        if (testLib.load()) {
            typedef void (*TasInitialize)(void);
            TasInitialize initFunction = (TasInitialize)testLib.resolve("qt_testability_init");
            if (initFunction) {
                initFunction();
            } else {
                qCritical("Could not resovle qttestabiltiy library!");
            }
        } else {
            qCritical("Failed to load qttestability!");
        }
    }

    view.engine()->rootContext()->setContextProperty("uriArgs",
            parser.positionalArguments());

    QString qmlFile;
    const QString filePath = QLatin1String("telegram.qml");
    QStringList paths = QStandardPaths::standardLocations(
            QStandardPaths::DataLocation);
    paths.prepend(QDir::currentPath());
    paths.prepend(QCoreApplication::applicationDirPath());
    Q_FOREACH(const QString &path, paths) {
        QString p = path + QLatin1Char('/') + filePath;
        if (QFile::exists(p)) {
            qmlFile = p;
            break;
        }
    }

    if (qmlFile.isEmpty()) {
        qFatal("Could not find %s!", qPrintable(filePath));
    }

    QGuiApplication::setApplicationName("Telegram");
    QGuiApplication::setOrganizationName("com.ubuntu.telegram");

    const QString cachePath = QStandardPaths::standardLocations(
            QStandardPaths::CacheLocation).first();
    QDir cache(cachePath);
    if (!cache.exists()) {
        qDebug() << "creating cache: " << cache.absolutePath();
        cache.mkpath(cache.absolutePath());
    }

    qDebug() << "launching " << qmlFile;
    view.setSource(QUrl::fromLocalFile(qmlFile));
    view.show();

    return app.exec();
}
