import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Telephony.PhoneNumber 0.1
import "../components"
import "../components/TelegramColors.js" as TelegramColors
import "../components/listitems"
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time
import Telegram 0.1

TelegramPage {
    property bool addManually: false
    property int peerId: 0
    property variant contact: undefined
    property real checkPhoneRequestId: 0

    property alias photo: photoImage.source
    property alias lastSeen: lastSeenText.text
    property alias phone: phoneNumberText.text
    property alias firstName: firstNameTextField.text
    property alias lastName: lastNameTextField.text
    property alias error: errorTextField.text

    signal addContactsAnswer(variant contactIds);
    signal getUserAnswer(ContactItem user);
    signal phoneCheckAnswer(var id, bool registered,bool invited)
    signal helpGetInviteTextAnswer(var id, string answerText)
    signal errorPhone(var id, var errorCode, string errorText)
    signal phoneCheckSent (var id ,string phoneNumberChecked)

    id: addContactPage
    title: i18n.tr("Add Contact");

    head.actions: [
            Action {
                iconName: "ok"
                text: i18n.tr("Done")
                onTriggered: addContact()
            },
            Action {
                iconName: "cancel"
                text: i18n.tr("Cancel")
                onTriggered: cancel()
            }
        ]

    body: Item {
        anchors.fill: parent
        anchors.margins: units.gu(2)

        Item {
            id: contactHeader
            anchors {
                top: parent.top
                left: parent.left
                right: parent.right
            }
            height: childrenRect.height
            visible: !addManually

            Image {
                id: photoImage
                anchors {
                    top: parent.top
                    left: parent.left
                }
                width: source == "" || source === undefined ? 0 : units.gu(8);
                height: width
                asynchronous: true
                fillMode: Image.PreserveAspectCrop
                source: "../images/user_blue.png"
            }

            Text {
                id: phoneNumberText
                anchors {
                    top: photoImage.top
                    left: photoImage.right
                    leftMargin: units.gu(2)
                }
                height: photoImage.height / 2.0
                verticalAlignment: TextInput.AlignVCenter
                horizontalAlignment: TextInput.AlignLeft

                font {
                    pixelSize: FontUtils.sizeToPixels("large")
                    bold: true
                }
                color: TelegramColors.black
                text: i18n.tr("+1 1234 56789")
            }

            Text {
                id: lastSeenText
                anchors {
                    left: photoImage.right
                    leftMargin: units.gu(2)
                    bottom: photoImage.bottom
                }
                height: photoImage.height / 2.0
                verticalAlignment: TextInput.AlignVCenter
                horizontalAlignment: TextInput.AlignLeft

                font {
                    pixelSize: FontUtils.sizeToPixels("medium")
                }
                color: TelegramColors.grey
                text: i18n.tr("last seen yesterday at 19:16")
            }
        }

        PhoneNumberField {
            id: phoneTextField
            anchors {
                top: parent.top
                topMargin: units.gu(4)
                left: parent.left
                right: parent.right
            }
            visible: addManually

            updateOnlyWhenFocused: false
            defaultRegion: "US"
            autoFormat: phoneTextField.text.length > 0
                        && phoneTextField.text.charAt(0) !== "*"
                        && phoneTextField.text.charAt(0) !== "#"
            inputMethodHints: Qt.ImhDialableCharactersOnly
            placeholderText: i18n.tr("Phone number (required)")

            Keys.onEnterPressed: firstNameTextField.forceActiveFocus()
            Keys.onReturnPressed: firstNameTextField.forceActiveFocus()
        }

        TextField {
            id: firstNameTextField
            anchors {
                top: addManually ? phoneTextField.bottom : contactHeader.bottom
                topMargin: addManually ? units.gu(2) : units.gu(4)
                left: parent.left
                right: parent.right
            }
            validator: RegExpValidator {
                regExp: /[\w\s]+/
            }
            placeholderText: i18n.tr("First name (required)")
            inputMethodHints: Qt.ImhNoPredictiveText
            Keys.onReturnPressed: lastNameTextField.forceActiveFocus()
        }

        TextField {
            id: lastNameTextField
            anchors {
                top: firstNameTextField.bottom
                topMargin: units.gu(2)
                left: parent.left
                right: parent.right
            }
            validator: RegExpValidator {
                regExp: /[\w\s]+/
            }
            placeholderText: i18n.tr("Last name (optional)")
            inputMethodHints: Qt.ImhNoPredictiveText
            Keys.onReturnPressed: addContact()
        }

        Text {
            id: errorTextField
            anchors {
                top: lastNameTextField.bottom
                topMargin: units.gu(2)
                left: parent.left
                right: parent.right
            }
            horizontalAlignment: Text.AlignHCenter
            color: "red"
        }
        Column {
            id: invite
            clip: true
            spacing: units.gu(1)
            property bool invited: false
            property string name: ""
            anchors {
                top: lastNameTextField.bottom
                topMargin: units.gu(4)
                left: parent.left
                right: parent.right
            }
            width: parent.width
            visible: false

            Label {
                anchors.horizontalCenter: parent.horizontalCenter
                text: i18n.tr("%1 is currently not in Telegram").arg(invite.name)
            }
            TelegramButton {
                anchors.horizontalCenter: parent.horizontalCenter
                text: i18n.tr("Invite to Telegram")
                onClicked: {
                    busy = true;
                    telegramClient.helpGetInviteTextAnswer.connect(
                            addContactPage.helpGetInviteTextAnswer);
                    telegramClient.helpGetInviteText(i18n.language);
                }
            }
            Label {
                anchors {
                    right: parent.right
                    margins: units.gu(1)
                }
                text: invited ? i18n.tr("%1 has a pending invite").arg(invite.name) : ""
                fontSize: "x-small"
            }

         }
        // Work around for lp:1382623
        Label {
            visible:invite.visible
            anchors {
                top: invite.bottom
                horizontalCenter: parent.horizontalCenter
                margins: units.gu(2.5)
            }
            text: i18n.tr("An invite text will be added to your Clipboard")
            fontSize: "x-small"
        }

    }

    Component.onCompleted: {
        telegramClient.addContactsAnswer.connect(addContactPage.addContactsAnswer);
        telegramClient.authCheckPhoneAnswer.connect(addContactPage.phoneCheckAnswer);
        telegramClient.error.connect(addContactPage.errorPhone);
        telegramClient.authCheckPhoneSent.connect(addContactPage.phoneCheckSent);

        if (!addManually) {
            telegramClient.getUserAnswer.connect(addContactPage.getUserAnswer);
            busy = true
            telegramClient.getUser(peerId);
        } else if (contact !== undefined) {
            // Contact imported from the address book. Note that the contact may
            // not be on Telegram. That is currently checked upon confirmation.
            updateFromContact(contact);
        }
    }

    Component.onDestruction: {
        if (!addManually) {
            telegramClient.getUserAnswer.disconnect(addContactPage.getUserAnswer);
        }
        telegramClient.error.disconnect(addContactPage.errorPhone);
        telegramClient.authCheckPhoneAnswer.disconnect(addContactPage.phoneCheckAnswer);
        telegramClient.addContactsAnswer.disconnect(addContactPage.addContactsAnswer);
    }

    onGetUserAnswer: {
        busy = false;
        if (user.id !== peerId) return;
        updateFromContact(user);
    }

    onAddContactsAnswer: {
        busy = false;
        if (contactIds.length === 0) {
            error = i18n.tr("Could not add contact");
        } else if (contactIds.length === 1) {
            error = "";
            pageStack.pop();
        }
    }

    onHelpGetInviteTextAnswer: {
        telegramClient.helpGetInviteTextAnswer.disconnect(
                addContactPage.helpGetInviteTextAnswer);

        // TODO can't add the invite text due to hint bug
        // https://bugs.launchpad.net/messaging-app/+bug/1382623
        // Remove add to clipboard when possible.
        Clipboard.push(answerText);

        telegramClient.authSendInvite(phone, answerText)
        Qt.openUrlExternally("message:///"+phone);
        busy = false;
        pageStack.pop();
    }

    onPhoneCheckSent: {
        if (phoneNumberChecked === phone) {
            checkPhoneRequestId = id;
        }
    }

    onPhoneCheckAnswer: {
        if ( id !== checkPhoneRequestId) return;
        if (registered) {
            telegramClient.addContact(firstName, lastName, phone);
        } else {
            busy = false;
            invite.name = firstName;
            invite.invited = invited;
            head.actions = [];
            invite.visible = true;
        }
    }
    onErrorPhone: {
        if (errorText == "PHONE_NUMBER_INVALID") {
            busy = false;
            error = i18n.tr("Number is not a valid phone number");
        }
    }

    /*
      Note: this function takes either Address Book or Telegram contact as argument!
    */    
    function updateFromContact(contact) {
        phone = contact.phone;
        if (!addManually) {
            if (contact.hasOwnProperty("thumbnail") && contact.thumbnail.length > 0) {
                photo = contact.thumbnail;
            } else if (contact.hasOwnProperty("id")) {
                photo = "../images/" + Avatar.getAvatar(contact.id);
            }
        }

        if (addManually) {
            phoneTextField.text = contact.phone;
        } else {
            phone = contact.phone;
        }

        if (contact.hasOwnProperty("online") && contact.online) {
            lastSeen = i18n.tr("online");
        } else if (contact.hasOwnProperty("lastSeenOnline")) {
            var lastSeenTime = Time.formatLastSeen(contact.lastSeenOnline * 1000);
            lastSeen = i18n.tr("last seen %1").arg(lastSeenTime);
        }

        firstName = contact.firstName;
        lastName = contact.lastName;
    }

    function addContact() {
        Qt.inputMethod.commit();
        error = "";
        if(!addContactPage.isConnected) {
            error = i18n.tr("Not connected, try again later");
            return;
        }
        var phoneToAdd = addManually ? phoneTextField.text : phoneNumberText.text;
        if (firstName.length > 0) {
            if (phoneToAdd.length > 0) {
                Qt.inputMethod.hide();
                phone = phoneToAdd;
                telegramClient.authCheckPhone(phoneToAdd);
                busy = true;
            } else {
                console.warn("Phone number is required. '" + phoneTextField.text + "'");
            }
        } else {
            console.warn("First name is required. '" + firstName + "'");
        }
    }

    function cancel() {
        pageStack.pop();
    }
}
