import QtQuick 2.0
import Ubuntu.Components 0.1
import Telegram 0.1
import "../components"
import "../components/TelegramColors.js" as TelegramColors
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time

TelegramPage {
    property int peerId: 0
    property int peerType: TLInputPeer.TypeInputPeerChat
    property bool isChat: peerType === TLInputPeer.TypeInputPeerChat
    property alias messagesModel: messagesModel

    signal getUserAnswer(ContactItem user);
    signal addContactsAnswer(var contactIds);

    id: dialogPage

    head.actions: [
        Action {
            iconName: "stock_image"
            text: i18n.tr("Send picture")
            enabled: isConnected
            onTriggered: requestPicture()
        },
        Action {
            iconName: "stock_video"
            text: i18n.tr("Send video")
            enabled: isConnected
            onTriggered: requestVideo()
        },
        Action {
            iconName: "stock_contact"
            text: isChat ? i18n.tr("Group Info") : i18n.tr("Profile Info")
            onTriggered: isChat ? openGroupProfile(peerId) : openProfile(peerId)
        }
    ]

    body: Item {

        anchors {
            fill: parent
        }

        Image {
            anchors.fill: parent
            source: Qt.resolvedUrl("../images/background_hd.jpg");
            fillMode: Image.Tile

            Component.onCompleted: {
                // Set size here, so we don't rescale on input method.
                sourceSize.height = parent.height;
                sourceSize.width = height * sourceSize.width / sourceSize.height;
            }

        }

        Rectangle {
            id: addContactHeader
            anchors {
                top: parent.top
                left: parent.left
                right: parent.right
            }
            height: units.gu(5)
            color: "#44000000"
            visible: peerType === TLInputPeer.TypeInputPeerForeign
            z: 1

            Text {
                id: addContactText
                anchors.centerIn: parent
                color: TelegramColors.white
                font {
                    pixelSize: FontUtils.sizeToPixels("small")
                    bold: true
                }
                text: i18n.tr("Add to contacts")
            }

            MouseArea {
                anchors.fill: parent
                onClicked: addContact()
            }
        }

        Rectangle {
            id: bottomRectangle
            anchors {
                left: parent.left
                right: parent.right
                bottom: parent.bottom
            }
            height: message.height + units.gu(2)

            color: "white"

            TextArea {
                id: message
                anchors {
                    left: parent.left
                    right: sendButtonBox.left
                    bottom: parent.bottom

                    margins: units.gu(1)
                }

                // this value is to avoid letter and underline being cut off
                height: units.gu(4.3)
                enabled: isConnected
                placeholderText: i18n.tr("Type message")
                inputMethodHints: Qt.ImhNone

                autoSize: true
                maximumLineCount: 4
                Keys.onReturnPressed: {
                    if (userSettings.contents.sendByEnter && isConnected) {
                        Qt.inputMethod.commit();
                        if (message.text.length === 0) return;

                        sendMessage(message.text);
                    } else {
                        event.accepted = false;
                    }
                }
            }

            Item {
                id: sendButtonBox
                anchors {
                    top: parent.top
                    bottom: parent.bottom
                    right: parent.right
                    rightMargin: units.gu(1)
                }
                width: sendButton.width + units.gu(1.5)

                Image {
                    id: sendButton
                    height: parent.height - units.gu(1)
                    anchors.centerIn: parent
                    fillMode: Image.PreserveAspectFit
                    focus: false
                    enabled: isConnected
                    source: enabled ? "../images/ic_send.png" : "../images/ic_send_disabled.png"
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        Qt.inputMethod.commit();
                        if (message.text.length === 0) return;

                        sendMessage(message.text);
                        list.positionViewAtBeginning();
                    }
                }
            }
        }

        MessagesModel {
            id: messagesModel

            onCountIncreased: {
                telegramClient.messagesReadHistory(inputPeer);
            }

            Component.onCompleted: messagesModel.setTelegramClient(telegramClient)
        }

        ListView {
            id: list
            verticalLayoutDirection: ListView.BottomToTop
            anchors {
                top: parent.top
                left: parent.left
                bottom: bottomRectangle.top
                right: parent.right
            }
            header: Item {
                height: units.gu(2)
            }
            footer: Item {
                height: units.gu(1)
            }
            cacheBuffer: units.gu(10)*20
            highlightFollowsCurrentItem: false
            clip: true

            /*
            // Disabling section headers because of upstream bug.
            section.property: "sectionDate"
            section.criteria: ViewSection.FullString
            section.delegate: sectionHeading
            */

            model: messagesModel
            delegate: TelegramDelegate {
                outgoing: model.out
                isSent: model.sent
                isRead: !model.unread
                isAction: model.isAction

                actionType: model.actionType
                actionTitle: model.actionTitle
                actionUser: model.actionUser
                actionThumbnail: isAction ? model.thumbnail : ""

                //FIXME: When section headers upstream bug is resolved revet this with section headers.
                isNewDay: (index === list.count) || !Time.areSameDay((messagesModel.get(index+1).date * 1000),model.date*1000)

                messageId: model.id
                message: getMessageText(model.mediaType, outgoing)
                messageDate: model.date
                mediaType: model.mediaType
                senderColor: Avatar.getColor(model.fromId)
                senderName: getSenderName()
                senderDisplayName: (!isChat || outgoing) ? "" : getSenderName()
                senderImage: {
                    if (isChat && !outgoing) {
                        if (model.fromThumbnail === "") {
                            return Qt.resolvedUrl("../images/" + Avatar.getAvatar(model.fromId));
                        } else {
                            return model.fromThumbnail;
                        }
                    }
                    return "";
                }
                time: Time.formatTimeOnly(model.date * 1000)
                thumbnail: isAction ? "" : model.thumbnail
                photo: model.photo
                video: model.video
                isDownloading: model.downloading || false
                progress: model.downloadedPercentage || 0

                onProfileImageClicked: {
                    openProfile(model.fromId);
                }

                leftSideAction: Action {
                    iconName: "delete"
                    text: i18n.tr("Delete")
                    onTriggered: telegramClient.messagesDeleteMessages([messageId])
                }

                locked: !isConnected

                function getSenderName() {
                    if (model.fromUserType === TLUser.TypeUserEmpty || model.fromUserType === TLUser.TypeUserDeleted) {
                        return i18n.tr("DELETED");
                    } else {
                        return model.fromFirstName + " " + model.fromLastName;
                    }
                }

                function getMessageText(mediaType, outgoing) {
                    var type = delegateUtils.getMediaTypeString(mediaType);
                    switch (true) {
                    case type === "Text":
                        return model.text;
                    case type === "Photo":
                        return "";
                    case type === "Video":
                        return ""
                    default:
                        return "Attached: " + type;
                    }
                }
            }

            Scrollbar {
                flickableItem: list
            }
        }

        DelegateUtils {
            id: delegateUtils
        }
    }

    PictureImport {
        id: pictureImporter

        onPictureReceived: {
            var filePath = String(pictureUrl).replace('file://', '')
            dialogPage.sendPhoto(filePath);
            message.forceActiveFocus()
        }
    }

    VideoImport {
        id: videoImporter

        onMediaReceived: {
            var filePath = String(mediaUrl).replace('file://', '')
            dialogPage.sendVideo(filePath);
            message.forceActiveFocus()
        }
    }

    Component.onCompleted: {
        updateChat(peerId, peerType);
        telegramClient.getUserAnswer.connect(dialogPage.getUserAnswer);
        telegramClient.addContactsAnswer.connect(dialogPage.addContactsAnswer);
    }

    Component.onDestruction: {
        telegramClient.getUserAnswer.disconnect(dialogPage.getUserAnswer);
        telegramClient.addContactsAnswer.disconnect(dialogPage.addContactsAnswer);
    }

    onAddContactsAnswer: {
        var contactAdded = false;
        for (var i = 0; i < contactIds.length; i++) {
            var contactId = contactIds[i];
            if (contactId === peerId) {
                peerType = TLInputPeer.TypeInputPeerContact;
                inputPeer.classType = TLInputPeer.TypeInputPeerContact;
                contactAdded = true;
                break;
            }
        }
        if (!contactAdded) {
            console.warn("Contacts added, but requested contact not.")
        }
    }

    function updateChat(peerId, peerType) {
        // Setup peer.
        inputPeer.classType = peerType;
        if (TLInputPeer.TypeInputPeerChat === peerType) {
            isChat = true;
            inputPeer.chatId = peerId;
        } else {
            isChat = false;
            inputPeer.userId = peerId;
        }
        // Setup list model.
        messagesModel.dialogId = peerId;
        messagesModel.peerType = peerType;
        messagesModel.updateQuery();
        messagesModel.fetchMore();
        // TODO Uncomment these when we resolve the "not a method" problem.
        //        var tags = [ peerId ];
        //        pushClient.clearPersistent(tags);
    }

    function requestPicture() {
        Qt.inputMethod.hide();
        pictureImporter.requestNewPicture();
    }

    function requestVideo() {
        Qt.inputMethod.hide();
        videoImporter.requestMedia();
    }

    function sendMessage(text) {
        if (text.length === 0) return;
        telegramClient.messagesSendMessage(inputPeer, text);
        message.text = "";
    }

    Timer {
        id: sendPhotoTimer

        property int attempts: 1
        property string photo: ""

        function send(photoPath) {
            stop();
            photo = photoPath;
            attempts = 0;
            restart();
        }

        interval: 750
        repeat: true
        onTriggered: {
            if (telegramClient.connected) {
                console.log("Connected, sending photo.");
                telegramClient.messagesSendPhoto(inputPeer, photo);
                stop();
            } else {
                attempts += 1;
                if (attempts >= 4) {
                    console.log("Picture not sent, giving up!");
                    stop();
                } else {
                    console.log("Picture not sent, will re-try. (" + attempts + ")");
                }
            }
        }
    }

    function sendPhoto(photo) {
        if (photo.length === 0) return;
        sendPhotoTimer.send(photo);
        // TODO Instead, this should be handled by qml-plugin. Queue the upload.
        // telegramClient.messagesSendPhoto(inputPeer, photo);
    }

    Timer {
        id: sendVideoTimer

        property int attempts: 1
        property string video: ""

        function send(videoPath) {
            stop();
            video = videoPath;
            attempts = 0;
            restart();
        }

        interval: 750
        repeat: true
        onTriggered: {
            if (telegramClient.connected) {
                console.log("Connected, sending video.");
                telegramClient.sendVideo(peerId, video);
                stop();
            } else {
                attempts += 1;
                if (attempts >= 4) {
                    console.log("Video not sent, giving up!");
                    stop();
                } else {
                    console.log("Video not sent, will re-try. (" + attempts + ")");
                }
            }
        }
    }

    function sendVideo(video) {
        if (video.length === 0) return;
        sendVideoTimer.send(video);
    }

    function addContact() {
        if (isChat) {
            console.warn("Adding single contact supported only.");
            return;
        }
        showAddContactPage(false, peerId);
    }

    function addMember() {
        if (peerType !== TLInputPeer.TypeInputPeerChat) {
            console.warn("Trying to add member to a non-group chat!");
            return;
        }

        var properties = { addToGroupMode: true, groupChatId: peerId, groupChatTitle: title };
        pageStack.push(contactsPage, properties);
    }

    TLInputPeer {
        id: inputPeer
    }
}
