import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0
import "../components"
import "../components/listitems"
import "../js/time.js" as Time
import Telegram 0.1

TelegramPage {
    id: dialogsPage

    head.actions: [
            Action {
                iconName: "search"
                text: i18n.tr("Search")
                visible: !isSearching
                onTriggered: searchPressed()
            },
            Action {
                iconName: "compose"
                text: i18n.tr("Compose")
                enabled: isConnected
                onTriggered: newChat()
            },
            Action {
                iconName: "contact-group"
                text: i18n.tr("New Group")
                enabled: isConnected
                onTriggered: newGroupChat()
            },
            Action {
                iconName: "contact"
                text: i18n.tr("Contacts")
                onTriggered: newChat()
            },
            Action {
                iconName: "settings"
                text: i18n.tr("Settings")
                onTriggered: openSettings()
            }
        ]

    head.backAction.visible: isSearching

    body: Item {
        anchors.fill: parent

        Label {
            id: listEmptyLabel
            anchors.centerIn: parent
            fontSize: "medium"
            visible: dialogsListView.count === 0
            text: !isOnline ? i18n.tr("Disconnected")
                            : isConnected ? i18n.tr("No chats") : i18n.tr("Waiting for connection...")
            z: 0
        }

        DialogsProxy {
            id: dialogsProxy
            sourceModel: dialogsModel
            sortRole: DialogsModel.TopMessageDateRole
            dynamicSortFilter: true
            ascending: false
            searchTerm: dialogsPage.searchTerm
        }

        ListView {
            id: dialogsListView
            anchors.fill: parent
            clip: true
            z: 1

            model: dialogsProxy
            delegate: TelegramDialogsListItem {
                peerId: model.id
                peerType: model.peerType

                isOutgoing: model.topMessageOut
                isSent: model.topMessageSent
                isRead: !model.topMessageUnread
                isAction: model.topMessageIsAction

                actionType: model.topMessageActionType
                actionTitle: model.topMessageActionTitle
                actionUser: model.topMessageActionUser

                isTyping: false//model.isTyping
                message: model.topMessageText
                thumbnail: model.thumbnail
                mediaType: model.topMessageMediaType

                title: model.name
                senderName: model.topMessageFromFirstName
                unreadCount: model.unreadCount
                messageDate: model.topMessageDate

                onItemClicked: {
                    searchFinished();
                    openChat(model.name, model.id, model.peerType);
                }
            }
        }

        Scrollbar {
            flickableItem: dialogsListView
        }

        DelegateUtils {
            id: delegateUtils
        }
    }
}
