import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import Ubuntu.Contacts 0.1
import Ubuntu.Components.Popups 0.1
import "../components"
import "../js/avatar.js" as Avatar
import "../components/TelegramColors.js" as TelegramColors
import "../js/time.js" as Time
import Telegram 0.1

TelegramPage {
    id: groupProfilePage

    property int chatId

    GroupModel {
        id: groupModel

        onGroupDetailsRefreshed: {
            console.log("yup onGroupDetailsRefreshed update");
            membersModel.setup(chatId);
        }
    }

    GroupMembersModel {
        id: membersModel
    }

    function addMember() {
        var properties = {
            addToGroupMode: true,
            groupChatId: groupModel.chatId,
            groupChatTitle: groupModel.title
        };
        pageStack.push(contactsPage, properties);
    }

    function openChat(contact) {
        pageStack.pop(); // ProfilePage
        pageStack.pop(); // GroupChat
        root.openChat(contact.name, contact.id,
                      membersModel.getInputPeerType(contact.peerType));
    }

    title: i18n.tr("Group Info")

    head.actions: [
        Action {
            iconName: "contact-new"
            text: i18n.tr("Add member")
            enabled: isConnected
            onTriggered: addMember()
        }
    ]

    body: Item {
        anchors.fill: parent

        Row {
            id: contactHeaderRow
            anchors {
                top: parent.top
                left: parent.left
                right: parent.right
                margins: units.gu(2)
            }
            height: units.gu(9)
            spacing: units.gu(1)

            Image {
                id: photoImage
                width: source == "" || source === undefined ? 0 : units.gu(9);
                height: width
                asynchronous: true
                fillMode: Image.PreserveAspectCrop
                source: groupModel.photo
            }

            Column {
                id: contactDetails
                anchors {
                    left: photoImage.right
                    right: parent.right
                    margins: units.gu(1)
                    verticalCenter: photoImage.verticalCenter
                }

                Text {
                    id: contactNameText
                    anchors {
                        left: parent.left
                        right: parent.right
                    }
                    maximumLineCount: 2
                    font.bold: true
                    font.pixelSize: FontUtils.sizeToPixels("large")
                    color: TelegramColors.black
                    elide: Text.ElideRight
                    wrapMode: "WordWrap"
                    text: groupModel.title
                }

/*
// Intentionally left commented. Will use when we support 'Notifications' or 'Shared media'.
                Label {
                    id: lastSeenText
                    color: TelegramColors.grey
                    text: groupModel.count + " " + i18n.tr("members");
                }
*/
            }
        }

        ListItem.Header {
            id: memberListHeader
            text: groupModel.count + " " + i18n.tr("members");
            anchors.top: contactHeaderRow.bottom
            anchors.topMargin: units.gu(2)
        }

/*
// Intentionally left commented.
            Component {
                id: photoActionsComponent

                Popover {
                    id: popover
                    Column {
                        id: containerLayout
                        anchors {
                            left: parent.left
                            top: parent.top
                            right: parent.right
                        }
                        ListItem.Header { text: i18n.tr("Group photo") }
                        ListItem.Standard { text: i18n.tr("Open photo") }
                        ListItem.Standard { text: i18n.tr("From camera") }
                        ListItem.Standard { text: i18n.tr("From gallery") }
                        ListItem.Standard { text: i18n.tr("Delete photo") }
                    }
                }
            }

            MouseArea {
                anchors.fill: parent
                onClicked: PopupUtils.open(photoActionsComponent, photoImage)
            }
*/

        ListView {
            id: members
            anchors {
                top: memberListHeader.bottom
                topMargin: units.gu(1)
                left: parent.left
                bottom: parent.bottom
                right: parent.right
            }
            footer: Item {
                height: units.gu(1)
            }
            highlightFollowsCurrentItem: false
            clip: true

            model: membersModel
            delegate: TelegramContactsListItem {
                id: delegate

                image: model.photo !== "" ? model.photo
                                          : Qt.resolvedUrl("../images/" + Avatar.getAvatar(model.id))
                title: model.name
                lastSeen: Time.formatLastSeen(model.lastSeenOnline * 1000)
                isOnline: model.online

                onClicked: openChat(model)
            }
        }
    }

    Component.onCompleted: {
        groupModel.setTelegramClient(telegramClient);
        groupModel.setChatId(chatId);
    }
}
