import QtQuick 2.0
import Ubuntu.Components 0.1
import "../components"

Page {
    property alias startMessagingButton: startMessaging
    property alias currentIndex: slider.currentIndex

    signal showSignInPage

    id: page

    IntroPageListModel {
        id: introModel
    }

    Rectangle {
        anchors.fill: parent
        color: "#ffffff"

        CrossFadeImage {
            id: icon
            anchors.horizontalCenter: parent.horizontalCenter
            y: parent.height/8 * 2
            z: 2
            width: Math.min(parent.width/3, units.gu(18))
            height: width

            fadeDuration: 250
            fillMode: Image.PreserveAspectFit
        }

        ListView {
            id: slider
            anchors.fill: parent
            z: 1

            orientation: ListView.Horizontal
            snapMode: ListView.SnapOneItem
            boundsBehavior: Flickable.StopAtBounds
            cacheBuffer: width
            clip: true
            highlightFollowsCurrentItem: true
            highlightRangeMode: ListView.StrictlyEnforceRange

            model: introModel
            delegate: Rectangle {
                id: delegateRect
                width: slider.width
                height: slider.height

                Rectangle {
                    width: parent.width
                    height: units.gu(10)
                    anchors.horizontalCenter: delegateRect.horizontalCenter
                    y: parent.height/6 * 3

                    // Problem changing Label color. Hence, using Text instead.
                    Text {
                        id: titleText
                        anchors.top: parent.top
                        width: parent.width

                        horizontalAlignment: TextInput.AlignHCenter
                        font.pixelSize: FontUtils.sizeToPixels("large")
                        color: "#000000"
                        text: title
                    }
                    // Not using Ubuntu font color, because it's affected by background color.
                    Text {
                        id: bodyText
                        anchors {
                            top: titleText.bottom
                            topMargin: units.gu(1.5)
                            // Why don't these work?
                            leftMargin: units.gu(15)
                            rightMargin: units.gu(3)
                        }
                        width: parent.width

                        horizontalAlignment: TextInput.AlignHCenter
                        font.pixelSize: FontUtils.sizeToPixels("medium")
                        wrapMode: Text.WordWrap
                        // Using Ubuntu Theme color.
                        color: "#777777"
                        text: body
                    }
                }
            }

            onCurrentIndexChanged: {
                icon.source = "../images/intro" + currentIndex + ".png";
            }
        }

        // TODO karni: Perhaps replace with styled Button. This doesn't work in Button, though:
        // font: Qt.font({family: "Ubuntu", pixelSize: FontUtils.sizeToPixels("medium"), color: "#000000"})
        TelegramButton {
            id: startMessaging
            anchors.horizontalCenter: parent.horizontalCenter
            y: page.height/3 * 2
            z: 2
            text: i18n.tr("Start Messaging")
            onClicked: showSignInPage()
        }

        Row {
            id: indicator
            height: units.gu(6)
            spacing: units.dp(8)
            anchors {
                bottom: parent.bottom
                horizontalCenter: parent.horizontalCenter
            }
            z: 2

            Repeater {
                model: introModel.count
                delegate: Rectangle {
                    height: width
                    width: units.dp(6)
                    antialiasing: true
                    anchors.verticalCenter: parent.verticalCenter
                    color: slider.currentIndex == index ? "#2ca5e0" : "#bbbbbb"
                    Behavior on color {
                        ColorAnimation {
                            duration: UbuntuAnimation.FastDuration
                        }
                    }
                }
            }
        }
    }
}
