import QtQuick 2.0
import Ubuntu.Components 0.1
import "../components"
import "../components/TelegramColors.js" as TelegramColors
import Ubuntu.Content 0.1
import QtMultimedia 5.0
import Telegram 0.1

TelegramPage {
    property string senderName: ""
    property string photoPreviewSource: ""
    property string videoPreviewSource: ""

    property bool isPlaying: videoPreview.playbackState === MediaPlayer.PlayingState

    id: previewPage
    title: i18n.tr("From: ") + senderName

    head.actions: [

        Action { 
            iconName: "save"
            text: i18n.tr("Save")
            onTriggered: pageStack.push(picker, {
                "url": photoPreview.source,
                "handler": ContentHandler.Destination, 
                "contentType": ContentType.Pictures
            });
            visible: photoPreviewSource !== ""
        },
        Action {
            iconName: "share"
            text: i18n.tr("Share")
            onTriggered: pageStack.push(picker, {
                "url": photoPreview.source,
                "handler": ContentHandler.Share, 
                "contentType": ContentType.Pictures
            });
            visible: photoPreviewSource !== ""
        }

    ]

    body: Item {

        anchors {
            fill: parent
        }

        Rectangle {
            anchors.fill: parent
            color: TelegramColors.page_background
        }

        Image {
            id: photoPreview
            anchors.fill: parent
            sourceSize.width: width
            sourceSize.height: height
            fillMode: Image.PreserveAspectFit

            visible: photoPreviewSource !== ""
            source: photoPreviewSource
        }

        MediaPlayer {
            id: videoPreview
            autoLoad: true
            autoPlay: true
            source: videoPreviewSource !== "" ? videoPreviewSource : ""
        }

        VideoOutput {
            id: videoOutput
            anchors.fill: parent
            visible: videoPreviewSource !== ""
            source: videoPreview
        }

        MouseArea {
            anchors.fill: videoOutput
            onClicked: isPlaying ? videoPreview.pause() : videoPreview.play()
        }

/*
        WebView {
            id: videoPreview
            anchors.fill: parent
            visible: videoPreviewSource.length > 0

            Component.onCompleted: loadHtml(
                    "<html><body><video src=\"%1\" controls width=\"%2\"></video></body></html>"
                    .arg(Qt.resolvedUrl(videoPreviewSource)).arg(previewPage.width), "file:///")
        }
*/
    }
}
