import QtQuick 2.0
import Ubuntu.Components 1.1
import QtQuick.Layouts 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import "../components"
import "../components/TelegramColors.js" as TelegramColors
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time
import Telegram 0.1

TelegramPage {
    id: profilePage

    property int userId: 0
    property var userClass: undefined
    property bool userExists: true

    property alias lastSeen: lastSeenText.text

    signal getUserAnswer(ContactItem user);

    function getUserInfo() {
        telegramClient.getUser(userId);
    }

    function updateFromContact(user) {
        profilePage.userId = user.id;
        profilePage.userClass = user.classType;

        if (user.classType === TLUser.TypeUserEmpty || user.classType === TLUser.TypeUserDeleted) {
            userExists = false;
            contactNameText.text = i18n.tr("DELETED");
        } else { 
            contactNameText.text = user.firstName + " " + user.lastName;
        }
        if (user.phone !== "") {
            phone.text = "+" + user.phone;
        }

        if (user.hasOwnProperty("online") && user.online) {
            lastSeen = i18n.tr("online");
        } else if (user.hasOwnProperty("lastSeenOnline")) {
            var lastSeenTime = Time.formatLastSeen(user.lastSeenOnline * 1000);
            lastSeen = i18n.tr("last seen %1").arg(lastSeenTime);
        }

        if (user.hasOwnProperty("thumbnail") && user.thumbnail.length > 0) {
            photoImage.source = user.thumbnail;
        } else if (user.hasOwnProperty("id")) {
            photoImage.source = "../images/" + Avatar.getAvatar(user.id);
        }
    }

    function openSimpleChat(title, id) {
        while (pageStack.depth !== 1) {
            pageStack.pop();
        }
        if (userClass == TLUser.TypeUserContact) {
            openChat(title, id, TLInputPeer.TypeInputPeerContact);
        } else if (userClass == TLUser.TypeUserForeign
                   || userClass == TLUser.TypeUserRequest) {
            openChat(title, id, TLInputPeer.TypeInputPeerForeign);
        } else {
            console.warn("Wrong user class for opening chat: " + userClass);
        }

    }

    Component.onCompleted: {
        telegramClient.getUserAnswer.connect(profilePage.getUserAnswer);
        getUserInfo();
    }

    Component.onDestruction: {
        telegramClient.getUserAnswer.disconnect(profilePage.getUserAnswer);
    }

    onGetUserAnswer: {
        if (user.id !== userId) return;
        updateFromContact(user);
    }

    title: i18n.tr("Contact Info")

    body: Item {
        anchors.fill: parent

        Row {
            id: contactHeaderRow

            anchors {
                top: parent.top
                left: parent.left
                right: parent.right
                margins: units.gu(2)
            }

            spacing: units.gu(2)

            Image {
                id: photoImage
                width: source == "" || source === undefined ? 0 : units.gu(8);
                height: width
                asynchronous: true
                fillMode: Image.PreserveAspectCrop
                source: "../images/user_blue.png"

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        var properties = { "title": contactNameText.text,
                                "photoPreviewSource": photoImage.source };
                        pageStack.push(previewPage, properties);
                    }
                }
            }

            Column {
                id: contactDetails

                spacing: units.gu(1)
                width: parent.width - photoImage.width
                anchors.verticalCenter: photoImage.verticalCenter

                Label {
                    id: contactNameText
                    font.bold: true
                    fontSize: "large"
                    color: TelegramColors.black
                    text: i18n.tr("Unknown")
                }

                Label {
                    id: lastSeenText
                    color: TelegramColors.grey
                    text: i18n.tr("last seen - unknown")
                }
            }
        }

        ListItem.Header {
            id: phoneHeader
            text: i18n.tr("Phone Number")
            visible: phoneRow.visible
            anchors.top: contactHeaderRow.bottom
            anchors.topMargin: units.gu(2)
        }

        RowLayout {
            id: phoneRow

            anchors {
                left: parent.left
                leftMargin: units.gu(2)
                right: parent.right
                rightMargin: units.gu(2)
                top: phoneHeader.bottom
            }

            Label {
                id: phone
                text: i18n.tr("Unknown")
                Layout.fillWidth: true
            }

            AbstractButton {
                id: callAction
                width: units.gu(5)
                height: width
                visible: phone.text !== i18n.tr("Unknown")
                Icon {
                    width: units.gu(4)
                    height: width
                    anchors.centerIn: parent
                    source: Qt.resolvedUrl("../images/call.png")
                }
                onClicked: {
                    Qt.openUrlExternally("tel:///" + phone.text)
                }
            }

            AbstractButton {
                id: msgAction
                width: units.gu(5)
                height: width
                visible: userExists
                Icon {
                    width: units.gu(4)
                    height: width
                    anchors.centerIn: parent
                    source: Qt.resolvedUrl("../images/ic_profile_send_message.png")
                }
                onClicked: {
                    openSimpleChat(contactNameText.text, userId)
                }
            }
        }
    }
}
