import QtQuick 2.0
import Ubuntu.Components 0.1
import "../components"

TelegramPage {
    property alias errorlabel: errorLabel
    property alias firstNameTextField: firstName
    property alias lastNameTextField: lastName

    title: isConnected ? i18n.tr("Sign up") : i18n.tr("Offline")
    onlineIndicationOnly: true

    body: Item {
        anchors {
            fill: parent
            margins: units.gu(2)
        }

        TelegramLabel {
            id: infoLabel
            anchors {
                top: parent.top
                margins: units.gu(1)
            }
            width: parent.width
            text: i18n.tr("Your first and last name:")
        }

        TelegramLabel {
            id: errorLabel
            anchors {
                top: infoLabel.bottom
                margins: units.gu(1)
            }
            width: parent.width
            visible: false
            color: "red"
        }

        TgTextField {
            id: firstName
            anchors {
                top: errorLabel.bottom
                topMargin: units.gu(1)
            }

            validator: RegExpValidator {
                regExp: /[\w\s]+/
            }
            placeholderText: i18n.tr("First name (required)")
            inputMethodHints: Qt.ImhNoPredictiveText

            onDisplayTextChanged: clearError()
            Keys.onReturnPressed: lastName.forceActiveFocus()
        }

        TgTextField {
            id: lastName
            anchors {
                top: firstName.bottom
                topMargin: units.gu(1)
            }
            placeholderText: i18n.tr("Last name (optional)")
            validator: RegExpValidator{ regExp: /[\w\s]+/ }
            inputMethodHints: Qt.ImhNoPredictiveText

            Keys.onReturnPressed: done()
        }

        TelegramButton {
            id: doneButton
            anchors {
                top: lastName.bottom
                topMargin: units.gu(1)
                left: parent.left
                right: parent.right
            }
            enabled: isConnected && firstName.text.length > 0
            text: i18n.tr("Sign up")
            onClicked: done()
        }
    }

    function onBackPressed() {
        signInPage.enableForm();
    }

    function done() {
        Qt.inputMethod.commit();
        Qt.inputMethod.hide();

        userData.firstName = firstName.text;
        userData.lastName = lastName.text;

        if (firstName.text.length > 0) {
            setFormEnabled(false);
            telegramClient.authSendCode();
        } else {
            onError(i18n.tr("First name is required"))
        }
    }

    // TODO Possibly refactor these out from Sign*Page(s).qml

    function onError(errorMessage) {
        setError(errorMessage);
        setFormEnabled(true);
    }

    function setError(errorMessage) {
        errorLabel.text = errorMessage;
        errorLabel.visible = true;
    }

    function setFormEnabled(enabled) {
        firstName.enabled = enabled;
        lastName.enabled = enabled;
    }

    function clearError() {
        if (errorLabel.visible) {
            errorLabel.visible = false;
            errorLabel.text = "";
        }
    }
}
